@@ -1,29 +1,22 @@
-extern crate num_bigint;
-extern crate num_integer;
-extern crate num_traits;
-#[cfg(feature = "rand")]
-extern crate rand;
-
 use num_bigint::BigUint;
-use num_bigint::{BigInt, ToBigInt};
 use num_bigint::Sign::{Minus, NoSign, Plus};
+use num_bigint::{BigInt, ToBigInt};
 
-use std::cmp::Ordering::{Less, Equal, Greater};
-use std::{f32, f64};
-use std::{i8, i16, i32, i64, isize};
-use std::iter::repeat;
-use std::{u8, u16, u32, u64, usize};
-#[cfg(has_i128)]
-use std::{i128, u128};
-use std::ops::Neg;
-use std::hash::{BuildHasher, Hasher, Hash};
+use std::cmp::Ordering::{Equal, Greater, Less};
 use std::collections::hash_map::RandomState;
+use std::hash::{BuildHasher, Hash, Hasher};
+use std::iter::repeat;
+use std::ops::Neg;
+use std::{f32, f64};
+use std::{i128, u128};
+use std::{i16, i32, i64, i8, isize};
+use std::{u16, u32, u64, u8, usize};
 
 use num_integer::Integer;
-use num_traits::{Zero, One, Signed, ToPrimitive, FromPrimitive, Num, Float};
+use num_traits::{pow, FromPrimitive, Num, One, Pow, Signed, ToPrimitive, Zero};
 
 mod consts;
-use consts::*;
+use crate::consts::*;
 
 #[macro_use]
 mod macros;
@@ -31,15 +24,17 @@ mod macros;
 #[test]
 fn test_from_bytes_be() {
     fn check(s: &str, result: &str) {
-        assert_eq!(BigInt::from_bytes_be(Plus, s.as_bytes()),
-                   BigInt::parse_bytes(result.as_bytes(), 10).unwrap());
+        assert_eq!(
+            BigInt::from_bytes_be(Plus, s.as_bytes()),
+            BigInt::parse_bytes(result.as_bytes(), 10).unwrap()
+        );
     }
     check("A", "65");
     check("AA", "16705");
     check("AB", "16706");
     check("Hello world!", "22405534230753963835153736737");
-    assert_eq!(BigInt::from_bytes_be(Plus, &[]), Zero::zero());
-    assert_eq!(BigInt::from_bytes_be(Minus, &[]), Zero::zero());
+    assert_eq!(BigInt::from_bytes_be(Plus, &[]), BigInt::zero());
+    assert_eq!(BigInt::from_bytes_be(Minus, &[]), BigInt::zero());
 }
 
 #[test]
@@ -64,15 +59,17 @@ fn test_to_bytes_be() {
 #[test]
 fn test_from_bytes_le() {
     fn check(s: &str, result: &str) {
-        assert_eq!(BigInt::from_bytes_le(Plus, s.as_bytes()),
-                   BigInt::parse_bytes(result.as_bytes(), 10).unwrap());
+        assert_eq!(
+            BigInt::from_bytes_le(Plus, s.as_bytes()),
+            BigInt::parse_bytes(result.as_bytes(), 10).unwrap()
+        );
     }
     check("A", "65");
     check("AA", "16705");
     check("BA", "16706");
     check("!dlrow olleH", "22405534230753963835153736737");
-    assert_eq!(BigInt::from_bytes_le(Plus, &[]), Zero::zero());
-    assert_eq!(BigInt::from_bytes_le(Minus, &[]), Zero::zero());
+    assert_eq!(BigInt::from_bytes_le(Plus, &[]), BigInt::zero());
+    assert_eq!(BigInt::from_bytes_le(Minus, &[]), BigInt::zero());
 }
 
 #[test]
@@ -97,8 +94,12 @@ fn test_to_bytes_le() {
 #[test]
 fn test_to_signed_bytes_le() {
     fn check(s: &str, result: Vec<u8>) {
-        assert_eq!(BigInt::parse_bytes(s.as_bytes(), 10).unwrap().to_signed_bytes_le(),
-                   result);
+        assert_eq!(
+            BigInt::parse_bytes(s.as_bytes(), 10)
+                .unwrap()
+                .to_signed_bytes_le(),
+            result
+        );
     }
 
     check("0", vec![0]);
@@ -108,13 +109,16 @@ fn test_to_signed_bytes_le() {
     check("-100", vec![156]);
     check("-8388608", vec![0, 0, 0x80]);
     check("-192", vec![0x40, 0xff]);
+    check("128", vec![0x80, 0])
 }
 
 #[test]
 fn test_from_signed_bytes_le() {
     fn check(s: &[u8], result: &str) {
-        assert_eq!(BigInt::from_signed_bytes_le(s),
-                   BigInt::parse_bytes(result.as_bytes(), 10).unwrap());
+        assert_eq!(
+            BigInt::from_signed_bytes_le(s),
+            BigInt::parse_bytes(result.as_bytes(), 10).unwrap()
+        );
     }
 
     check(&[], "0");
@@ -132,8 +136,12 @@ fn test_from_signed_bytes_le() {
 #[test]
 fn test_to_signed_bytes_be() {
     fn check(s: &str, result: Vec<u8>) {
-        assert_eq!(BigInt::parse_bytes(s.as_bytes(), 10).unwrap().to_signed_bytes_be(),
-                   result);
+        assert_eq!(
+            BigInt::parse_bytes(s.as_bytes(), 10)
+                .unwrap()
+                .to_signed_bytes_be(),
+            result
+        );
     }
 
     check("0", vec![0]);
@@ -143,13 +151,16 @@ fn test_to_signed_bytes_be() {
     check("-100", vec![156]);
     check("-8388608", vec![128, 0, 0]);
     check("-192", vec![0xff, 0x40]);
+    check("128", vec![0, 0x80]);
 }
 
 #[test]
 fn test_from_signed_bytes_be() {
     fn check(s: &[u8], result: &str) {
-        assert_eq!(BigInt::from_signed_bytes_be(s),
-                   BigInt::parse_bytes(result.as_bytes(), 10).unwrap());
+        assert_eq!(
+            BigInt::from_signed_bytes_be(s),
+            BigInt::parse_bytes(result.as_bytes(), 10).unwrap()
+        );
     }
 
     check(&[], "0");
@@ -164,6 +175,22 @@ fn test_from_signed_bytes_be() {
     check(&[0xff, 0x40], "-192");
 }
 
+#[test]
+fn test_signed_bytes_be_round_trip() {
+    for i in -0x1FFFF..0x20000 {
+        let n = BigInt::from(i);
+        assert_eq!(n, BigInt::from_signed_bytes_be(&n.to_signed_bytes_be()));
+    }
+}
+
+#[test]
+fn test_signed_bytes_le_round_trip() {
+    for i in -0x1FFFF..0x20000 {
+        let n = BigInt::from(i);
+        assert_eq!(n, BigInt::from_signed_bytes_le(&n.to_signed_bytes_le()));
+    }
+}
+
 #[test]
 fn test_cmp() {
     let vs: [&[u32]; 4] = [&[2 as u32], &[1, 1], &[2, 1], &[1, 1, 1]];
@@ -260,7 +287,6 @@ fn test_convert_i64() {
 }
 
 #[test]
-#[cfg(has_i128)]
 fn test_convert_i128() {
     fn check(b1: BigInt, i: i128) {
         let b2: BigInt = FromPrimitive::from_i128(i).unwrap();
@@ -318,7 +344,6 @@ fn test_convert_u64() {
 }
 
 #[test]
-#[cfg(has_i128)]
 fn test_convert_u128() {
     fn check(b1: BigInt, u: u128) {
         let b2: BigInt = FromPrimitive::from_u128(u).unwrap();
@@ -345,6 +370,7 @@ fn test_convert_u128() {
 }
 
 #[test]
+#[allow(clippy::float_cmp)]
 fn test_convert_f32() {
     fn check(b1: &BigInt, f: f32) {
         let b2 = BigInt::from_f32(f).unwrap();
@@ -358,12 +384,14 @@ fn test_convert_f32() {
 
     check(&BigInt::zero(), 0.0);
     check(&BigInt::one(), 1.0);
-    check(&BigInt::from(u16::MAX), 2.0.powi(16) - 1.0);
-    check(&BigInt::from(1u64 << 32), 2.0.powi(32));
-    check(&BigInt::from_slice(Plus, &[0, 0, 1]), 2.0.powi(64));
-    check(&((BigInt::one() << 100) + (BigInt::one() << 123)),
-          2.0.powi(100) + 2.0.powi(123));
-    check(&(BigInt::one() << 127), 2.0.powi(127));
+    check(&BigInt::from(u16::MAX), pow(2.0_f32, 16) - 1.0);
+    check(&BigInt::from(1u64 << 32), pow(2.0_f32, 32));
+    check(&BigInt::from_slice(Plus, &[0, 0, 1]), pow(2.0_f32, 64));
+    check(
+        &((BigInt::one() << 100) + (BigInt::one() << 123)),
+        pow(2.0_f32, 100) + pow(2.0_f32, 123),
+    );
+    check(&(BigInt::one() << 127), pow(2.0_f32, 127));
     check(&(BigInt::from((1u64 << 24) - 1) << (128 - 24)), f32::MAX);
 
     // keeping all 24 digits with the bits at different offsets to the BigDigits
@@ -373,7 +401,7 @@ fn test_convert_f32() {
     for _ in 0..64 {
         check(&b, f);
         f *= 2.0;
-        b = b << 1;
+        b <<= 1;
     }
 
     // this number when rounded to f64 then f32 isn't the same as when rounded straight to f32
@@ -390,18 +418,22 @@ fn test_convert_f32() {
     for _ in 0..64 {
         assert_eq!(b.to_f32(), Some(f));
         f *= 2.0;
-        b = b << 1;
+        b <<= 1;
     }
 
     // rounding
-    assert_eq!(BigInt::from_f32(-f32::consts::PI),
-               Some(BigInt::from(-3i32)));
+    assert_eq!(
+        BigInt::from_f32(-f32::consts::PI),
+        Some(BigInt::from(-3i32))
+    );
     assert_eq!(BigInt::from_f32(-f32::consts::E), Some(BigInt::from(-2i32)));
     assert_eq!(BigInt::from_f32(-0.99999), Some(BigInt::zero()));
     assert_eq!(BigInt::from_f32(-0.5), Some(BigInt::zero()));
     assert_eq!(BigInt::from_f32(-0.0), Some(BigInt::zero()));
-    assert_eq!(BigInt::from_f32(f32::MIN_POSITIVE / 2.0),
-               Some(BigInt::zero()));
+    assert_eq!(
+        BigInt::from_f32(f32::MIN_POSITIVE / 2.0),
+        Some(BigInt::zero())
+    );
     assert_eq!(BigInt::from_f32(f32::MIN_POSITIVE), Some(BigInt::zero()));
     assert_eq!(BigInt::from_f32(0.5), Some(BigInt::zero()));
     assert_eq!(BigInt::from_f32(0.99999), Some(BigInt::zero()));
@@ -414,19 +446,29 @@ fn test_convert_f32() {
     assert_eq!(BigInt::from_f32(f32::NEG_INFINITY), None);
 
     // largest BigInt that will round to a finite f32 value
-    let big_num = (BigInt::one() << 128) - BigInt::one() - (BigInt::one() << (128 - 25));
+    let big_num = (BigInt::one() << 128u8) - 1u8 - (BigInt::one() << (128u8 - 25));
     assert_eq!(big_num.to_f32(), Some(f32::MAX));
-    assert_eq!((&big_num + BigInt::one()).to_f32(), None);
+    assert_eq!((&big_num + 1u8).to_f32(), Some(f32::INFINITY));
     assert_eq!((-&big_num).to_f32(), Some(f32::MIN));
-    assert_eq!(((-&big_num) - BigInt::one()).to_f32(), None);
+    assert_eq!(((-&big_num) - 1u8).to_f32(), Some(f32::NEG_INFINITY));
 
-    assert_eq!(((BigInt::one() << 128) - BigInt::one()).to_f32(), None);
-    assert_eq!((BigInt::one() << 128).to_f32(), None);
-    assert_eq!((-((BigInt::one() << 128) - BigInt::one())).to_f32(), None);
-    assert_eq!((-(BigInt::one() << 128)).to_f32(), None);
+    assert_eq!(
+        ((BigInt::one() << 128u8) - 1u8).to_f32(),
+        Some(f32::INFINITY)
+    );
+    assert_eq!((BigInt::one() << 128u8).to_f32(), Some(f32::INFINITY));
+    assert_eq!(
+        (-((BigInt::one() << 128u8) - 1u8)).to_f32(),
+        Some(f32::NEG_INFINITY)
+    );
+    assert_eq!(
+        (-(BigInt::one() << 128u8)).to_f32(),
+        Some(f32::NEG_INFINITY)
+    );
 }
 
 #[test]
+#[allow(clippy::float_cmp)]
 fn test_convert_f64() {
     fn check(b1: &BigInt, f: f64) {
         let b2 = BigInt::from_f64(f).unwrap();
@@ -440,12 +482,14 @@ fn test_convert_f64() {
 
     check(&BigInt::zero(), 0.0);
     check(&BigInt::one(), 1.0);
-    check(&BigInt::from(u32::MAX), 2.0.powi(32) - 1.0);
-    check(&BigInt::from(1u64 << 32), 2.0.powi(32));
-    check(&BigInt::from_slice(Plus, &[0, 0, 1]), 2.0.powi(64));
-    check(&((BigInt::one() << 100) + (BigInt::one() << 152)),
-          2.0.powi(100) + 2.0.powi(152));
-    check(&(BigInt::one() << 1023), 2.0.powi(1023));
+    check(&BigInt::from(u32::MAX), pow(2.0_f64, 32) - 1.0);
+    check(&BigInt::from(1u64 << 32), pow(2.0_f64, 32));
+    check(&BigInt::from_slice(Plus, &[0, 0, 1]), pow(2.0_f64, 64));
+    check(
+        &((BigInt::one() << 100) + (BigInt::one() << 152)),
+        pow(2.0_f64, 100) + pow(2.0_f64, 152),
+    );
+    check(&(BigInt::one() << 1023), pow(2.0_f64, 1023));
     check(&(BigInt::from((1u64 << 53) - 1) << (1024 - 53)), f64::MAX);
 
     // keeping all 53 digits with the bits at different offsets to the BigDigits
@@ -455,7 +499,7 @@ fn test_convert_f64() {
     for _ in 0..128 {
         check(&b, f);
         f *= 2.0;
-        b = b << 1;
+        b <<= 1;
     }
 
     // test rounding up with the bits at different offsets to the BigDigits
@@ -464,18 +508,22 @@ fn test_convert_f64() {
     for _ in 0..128 {
         assert_eq!(b.to_f64(), Some(f));
         f *= 2.0;
-        b = b << 1;
+        b <<= 1;
     }
 
     // rounding
-    assert_eq!(BigInt::from_f64(-f64::consts::PI),
-               Some(BigInt::from(-3i32)));
+    assert_eq!(
+        BigInt::from_f64(-f64::consts::PI),
+        Some(BigInt::from(-3i32))
+    );
     assert_eq!(BigInt::from_f64(-f64::consts::E), Some(BigInt::from(-2i32)));
     assert_eq!(BigInt::from_f64(-0.99999), Some(BigInt::zero()));
     assert_eq!(BigInt::from_f64(-0.5), Some(BigInt::zero()));
     assert_eq!(BigInt::from_f64(-0.0), Some(BigInt::zero()));
-    assert_eq!(BigInt::from_f64(f64::MIN_POSITIVE / 2.0),
-               Some(BigInt::zero()));
+    assert_eq!(
+        BigInt::from_f64(f64::MIN_POSITIVE / 2.0),
+        Some(BigInt::zero())
+    );
     assert_eq!(BigInt::from_f64(f64::MIN_POSITIVE), Some(BigInt::zero()));
     assert_eq!(BigInt::from_f64(0.5), Some(BigInt::zero()));
     assert_eq!(BigInt::from_f64(0.99999), Some(BigInt::zero()));
@@ -488,16 +536,25 @@ fn test_convert_f64() {
     assert_eq!(BigInt::from_f64(f64::NEG_INFINITY), None);
 
     // largest BigInt that will round to a finite f64 value
-    let big_num = (BigInt::one() << 1024) - BigInt::one() - (BigInt::one() << (1024 - 54));
+    let big_num = (BigInt::one() << 1024u16) - 1u8 - (BigInt::one() << (1024u16 - 54));
     assert_eq!(big_num.to_f64(), Some(f64::MAX));
-    assert_eq!((&big_num + BigInt::one()).to_f64(), None);
+    assert_eq!((&big_num + 1u8).to_f64(), Some(f64::INFINITY));
     assert_eq!((-&big_num).to_f64(), Some(f64::MIN));
-    assert_eq!(((-&big_num) - BigInt::one()).to_f64(), None);
+    assert_eq!(((-&big_num) - 1u8).to_f64(), Some(f64::NEG_INFINITY));
 
-    assert_eq!(((BigInt::one() << 1024) - BigInt::one()).to_f64(), None);
-    assert_eq!((BigInt::one() << 1024).to_f64(), None);
-    assert_eq!((-((BigInt::one() << 1024) - BigInt::one())).to_f64(), None);
-    assert_eq!((-(BigInt::one() << 1024)).to_f64(), None);
+    assert_eq!(
+        ((BigInt::one() << 1024u16) - 1u8).to_f64(),
+        Some(f64::INFINITY)
+    );
+    assert_eq!((BigInt::one() << 1024u16).to_f64(), Some(f64::INFINITY));
+    assert_eq!(
+        (-((BigInt::one() << 1024u16) - 1u8)).to_f64(),
+        Some(f64::NEG_INFINITY)
+    );
+    assert_eq!(
+        (-(BigInt::one() << 1024u16)).to_f64(),
+        Some(f64::NEG_INFINITY)
+    );
 }
 
 #[test]
@@ -532,8 +589,10 @@ fn test_convert_from_uint() {
     check!(u16, BigInt::from_slice(Plus, &[u16::MAX as u32]));
     check!(u32, BigInt::from_slice(Plus, &[u32::MAX]));
     check!(u64, BigInt::from_slice(Plus, &[u32::MAX, u32::MAX]));
-    #[cfg(has_i128)]
-    check!(u128, BigInt::from_slice(Plus, &[u32::MAX, u32::MAX, u32::MAX, u32::MAX]));
+    check!(
+        u128,
+        BigInt::from_slice(Plus, &[u32::MAX, u32::MAX, u32::MAX, u32::MAX])
+    );
     check!(usize, BigInt::from(usize::MAX as u64));
 }
 
@@ -548,36 +607,49 @@ fn test_convert_from_int() {
             assert_eq!(BigInt::from($ty::one()), BigInt::one());
             assert_eq!(BigInt::from($ty::MAX - $ty::one()), $max - BigInt::one());
             assert_eq!(BigInt::from($ty::MAX), $max);
-        }
+        };
     }
 
-    check!(i8,
-           BigInt::from_slice(Minus, &[1 << 7]),
-           BigInt::from_slice(Plus, &[i8::MAX as u32]));
-    check!(i16,
-           BigInt::from_slice(Minus, &[1 << 15]),
-           BigInt::from_slice(Plus, &[i16::MAX as u32]));
-    check!(i32,
-           BigInt::from_slice(Minus, &[1 << 31]),
-           BigInt::from_slice(Plus, &[i32::MAX as u32]));
-    check!(i64,
-           BigInt::from_slice(Minus, &[0, 1 << 31]),
-           BigInt::from_slice(Plus, &[u32::MAX, i32::MAX as u32]));
-    #[cfg(has_i128)]
-    check!(i128,
-           BigInt::from_slice(Minus, &[0, 0, 0, 1 << 31]),
-           BigInt::from_slice(Plus, &[u32::MAX, u32::MAX, u32::MAX, i32::MAX as u32]));
-    check!(isize,
-           BigInt::from(isize::MIN as i64),
-           BigInt::from(isize::MAX as i64));
+    check!(
+        i8,
+        BigInt::from_slice(Minus, &[1 << 7]),
+        BigInt::from_slice(Plus, &[i8::MAX as u32])
+    );
+    check!(
+        i16,
+        BigInt::from_slice(Minus, &[1 << 15]),
+        BigInt::from_slice(Plus, &[i16::MAX as u32])
+    );
+    check!(
+        i32,
+        BigInt::from_slice(Minus, &[1 << 31]),
+        BigInt::from_slice(Plus, &[i32::MAX as u32])
+    );
+    check!(
+        i64,
+        BigInt::from_slice(Minus, &[0, 1 << 31]),
+        BigInt::from_slice(Plus, &[u32::MAX, i32::MAX as u32])
+    );
+    check!(
+        i128,
+        BigInt::from_slice(Minus, &[0, 0, 0, 1 << 31]),
+        BigInt::from_slice(Plus, &[u32::MAX, u32::MAX, u32::MAX, i32::MAX as u32])
+    );
+    check!(
+        isize,
+        BigInt::from(isize::MIN as i64),
+        BigInt::from(isize::MAX as i64)
+    );
 }
 
 #[test]
 fn test_convert_from_biguint() {
     assert_eq!(BigInt::from(BigUint::zero()), BigInt::zero());
     assert_eq!(BigInt::from(BigUint::one()), BigInt::one());
-    assert_eq!(BigInt::from(BigUint::from_slice(&[1, 2, 3])),
-               BigInt::from_slice(Plus, &[1, 2, 3]));
+    assert_eq!(
+        BigInt::from(BigUint::from_slice(&[1, 2, 3])),
+        BigInt::from_slice(Plus, &[1, 2, 3])
+    );
 }
 
 #[test]
@@ -596,7 +668,7 @@ fn test_add() {
         assert_op!(a + nc == nb);
         assert_op!(b + nc == na);
         assert_op!(na + nb == nc);
-        assert_op!(a + na == Zero::zero());
+        assert_op!(a + na == BigInt::zero());
 
         assert_assign_op!(a += b == c);
         assert_assign_op!(b += a == c);
@@ -605,7 +677,7 @@ fn test_add() {
         assert_assign_op!(a += nc == nb);
         assert_assign_op!(b += nc == na);
         assert_assign_op!(na += nb == nc);
-        assert_assign_op!(a += na == Zero::zero());
+        assert_assign_op!(a += na == BigInt::zero());
     }
 }
 
@@ -625,7 +697,7 @@ fn test_sub() {
         assert_op!(b - na == c);
         assert_op!(a - nb == c);
         assert_op!(nc - na == nb);
-        assert_op!(a - a == Zero::zero());
+        assert_op!(a - a == BigInt::zero());
 
         assert_assign_op!(c -= a == b);
         assert_assign_op!(c -= b == a);
@@ -634,7 +706,7 @@ fn test_sub() {
         assert_assign_op!(b -= na == c);
         assert_assign_op!(a -= nb == c);
         assert_assign_op!(nc -= na == nb);
-        assert_assign_op!(a -= a == Zero::zero());
+        assert_assign_op!(a -= a == BigInt::zero());
     }
 }
 
@@ -678,6 +750,8 @@ fn test_mul() {
 fn test_div_mod_floor() {
     fn check_sub(a: &BigInt, b: &BigInt, ans_d: &BigInt, ans_m: &BigInt) {
         let (d, m) = a.div_mod_floor(b);
+        assert_eq!(d, a.div_floor(b));
+        assert_eq!(m, a.mod_floor(b));
         if !m.is_zero() {
             assert_eq!(m.sign(), b.sign());
         }
@@ -729,7 +803,6 @@ fn test_div_mod_floor() {
     }
 }
 
-
 #[test]
 fn test_div_rem() {
     fn check_sub(a: &BigInt, b: &BigInt, ans_q: &BigInt, ans_r: &BigInt) {
@@ -782,6 +855,53 @@ fn test_div_rem() {
     }
 }
 
+#[test]
+fn test_div_ceil() {
+    fn check_sub(a: &BigInt, b: &BigInt, ans_d: &BigInt) {
+        assert_eq!(a.div_ceil(b), *ans_d);
+    }
+
+    fn check(a: &BigInt, b: &BigInt, d: &BigInt, m: &BigInt) {
+        if m.is_zero() {
+            check_sub(a, b, d);
+            check_sub(a, &b.neg(), &d.neg());
+            check_sub(&a.neg(), b, &d.neg());
+            check_sub(&a.neg(), &b.neg(), d);
+        } else {
+            check_sub(a, b, &(d + 1));
+            check_sub(a, &b.neg(), &d.neg());
+            check_sub(&a.neg(), b, &d.neg());
+            check_sub(&a.neg(), &b.neg(), &(d + 1));
+        }
+    }
+
+    for elm in MUL_TRIPLES.iter() {
+        let (a_vec, b_vec, c_vec) = *elm;
+        let a = BigInt::from_slice(Plus, a_vec);
+        let b = BigInt::from_slice(Plus, b_vec);
+        let c = BigInt::from_slice(Plus, c_vec);
+
+        if !a.is_zero() {
+            check(&c, &a, &b, &Zero::zero());
+        }
+        if !b.is_zero() {
+            check(&c, &b, &a, &Zero::zero());
+        }
+    }
+
+    for elm in DIV_REM_QUADRUPLES.iter() {
+        let (a_vec, b_vec, c_vec, d_vec) = *elm;
+        let a = BigInt::from_slice(Plus, a_vec);
+        let b = BigInt::from_slice(Plus, b_vec);
+        let c = BigInt::from_slice(Plus, c_vec);
+        let d = BigInt::from_slice(Plus, d_vec);
+
+        if !b.is_zero() {
+            check(&a, &b, &c, &d);
+        }
+    }
+}
+
 #[test]
 fn test_checked_add() {
     for elm in SUM_TRIPLES.iter() {
@@ -797,7 +917,7 @@ fn test_checked_add() {
         assert!(a.checked_add(&(-&c)).unwrap() == (-&b));
         assert!(b.checked_add(&(-&c)).unwrap() == (-&a));
         assert!((-&a).checked_add(&(-&b)).unwrap() == (-&c));
-        assert!(a.checked_add(&(-&a)).unwrap() == Zero::zero());
+        assert!(a.checked_add(&(-&a)).unwrap() == BigInt::zero());
     }
 }
 
@@ -816,7 +936,7 @@ fn test_checked_sub() {
         assert!(b.checked_sub(&(-&a)).unwrap() == c);
         assert!(a.checked_sub(&(-&b)).unwrap() == c);
         assert!((-&c).checked_sub(&(-&a)).unwrap() == (-&b));
-        assert!(a.checked_sub(&a).unwrap() == Zero::zero());
+        assert!(a.checked_sub(&a).unwrap() == BigInt::zero());
     }
 }
 
@@ -878,6 +998,9 @@ fn test_gcd() {
         let big_c: BigInt = FromPrimitive::from_isize(c).unwrap();
 
         assert_eq!(big_a.gcd(&big_b), big_c);
+        assert_eq!(big_a.extended_gcd(&big_b).gcd, big_c);
+        assert_eq!(big_a.gcd_lcm(&big_b).0, big_c);
+        assert_eq!(big_a.extended_gcd_lcm(&big_b).0.gcd, big_c);
     }
 
     check(10, 2, 2);
@@ -898,8 +1021,11 @@ fn test_lcm() {
         let big_c: BigInt = FromPrimitive::from_isize(c).unwrap();
 
         assert_eq!(big_a.lcm(&big_b), big_c);
+        assert_eq!(big_a.gcd_lcm(&big_b).1, big_c);
+        assert_eq!(big_a.extended_gcd_lcm(&big_b).1, big_c);
     }
 
+    check(0, 0, 0);
     check(1, 0, 0);
     check(0, 1, 0);
     check(1, 1, 1);
@@ -910,6 +1036,78 @@ fn test_lcm() {
     check(11, 5, 55);
 }
 
+#[test]
+fn test_next_multiple_of() {
+    assert_eq!(
+        BigInt::from(16).next_multiple_of(&BigInt::from(8)),
+        BigInt::from(16)
+    );
+    assert_eq!(
+        BigInt::from(23).next_multiple_of(&BigInt::from(8)),
+        BigInt::from(24)
+    );
+    assert_eq!(
+        BigInt::from(16).next_multiple_of(&BigInt::from(-8)),
+        BigInt::from(16)
+    );
+    assert_eq!(
+        BigInt::from(23).next_multiple_of(&BigInt::from(-8)),
+        BigInt::from(16)
+    );
+    assert_eq!(
+        BigInt::from(-16).next_multiple_of(&BigInt::from(8)),
+        BigInt::from(-16)
+    );
+    assert_eq!(
+        BigInt::from(-23).next_multiple_of(&BigInt::from(8)),
+        BigInt::from(-16)
+    );
+    assert_eq!(
+        BigInt::from(-16).next_multiple_of(&BigInt::from(-8)),
+        BigInt::from(-16)
+    );
+    assert_eq!(
+        BigInt::from(-23).next_multiple_of(&BigInt::from(-8)),
+        BigInt::from(-24)
+    );
+}
+
+#[test]
+fn test_prev_multiple_of() {
+    assert_eq!(
+        BigInt::from(16).prev_multiple_of(&BigInt::from(8)),
+        BigInt::from(16)
+    );
+    assert_eq!(
+        BigInt::from(23).prev_multiple_of(&BigInt::from(8)),
+        BigInt::from(16)
+    );
+    assert_eq!(
+        BigInt::from(16).prev_multiple_of(&BigInt::from(-8)),
+        BigInt::from(16)
+    );
+    assert_eq!(
+        BigInt::from(23).prev_multiple_of(&BigInt::from(-8)),
+        BigInt::from(24)
+    );
+    assert_eq!(
+        BigInt::from(-16).prev_multiple_of(&BigInt::from(8)),
+        BigInt::from(-16)
+    );
+    assert_eq!(
+        BigInt::from(-23).prev_multiple_of(&BigInt::from(8)),
+        BigInt::from(-24)
+    );
+    assert_eq!(
+        BigInt::from(-16).prev_multiple_of(&BigInt::from(-8)),
+        BigInt::from(-16)
+    );
+    assert_eq!(
+        BigInt::from(-23).prev_multiple_of(&BigInt::from(-8)),
+        BigInt::from(-16)
+    );
+}
+
 #[test]
 fn test_abs_sub() {
     let zero: BigInt = Zero::zero();
@@ -950,14 +1148,16 @@ fn test_from_str_radix() {
 
     // issue 10522, this hit an edge case that caused it to
     // attempt to allocate a vector of size (-1u) == huge.
-    let x: BigInt = format!("1{}", repeat("0").take(36).collect::<String>()).parse().unwrap();
+    let x: BigInt = format!("1{}", repeat("0").take(36).collect::<String>())
+        .parse()
+        .unwrap();
     let _y = x.to_string();
 }
 
 #[test]
 fn test_lower_hex() {
     let a = BigInt::parse_bytes(b"A", 16).unwrap();
-    let hello = BigInt::parse_bytes("-22405534230753963835153736737".as_bytes(), 10).unwrap();
+    let hello = BigInt::parse_bytes(b"-22405534230753963835153736737", 10).unwrap();
 
     assert_eq!(format!("{:x}", a), "a");
     assert_eq!(format!("{:x}", hello), "-48656c6c6f20776f726c6421");
@@ -967,7 +1167,7 @@ fn test_lower_hex() {
 #[test]
 fn test_upper_hex() {
     let a = BigInt::parse_bytes(b"A", 16).unwrap();
-    let hello = BigInt::parse_bytes("-22405534230753963835153736737".as_bytes(), 10).unwrap();
+    let hello = BigInt::parse_bytes(b"-22405534230753963835153736737", 10).unwrap();
 
     assert_eq!(format!("{:X}", a), "A");
     assert_eq!(format!("{:X}", hello), "-48656C6C6F20776F726C6421");
@@ -977,18 +1177,20 @@ fn test_upper_hex() {
 #[test]
 fn test_binary() {
     let a = BigInt::parse_bytes(b"A", 16).unwrap();
-    let hello = BigInt::parse_bytes("-224055342307539".as_bytes(), 10).unwrap();
+    let hello = BigInt::parse_bytes(b"-224055342307539", 10).unwrap();
 
     assert_eq!(format!("{:b}", a), "1010");
-    assert_eq!(format!("{:b}", hello),
-               "-110010111100011011110011000101101001100011010011");
+    assert_eq!(
+        format!("{:b}", hello),
+        "-110010111100011011110011000101101001100011010011"
+    );
     assert_eq!(format!("{:♥>+#8b}", a), "♥+0b1010");
 }
 
 #[test]
 fn test_octal() {
     let a = BigInt::parse_bytes(b"A", 16).unwrap();
-    let hello = BigInt::parse_bytes("-22405534230753963835153736737".as_bytes(), 10).unwrap();
+    let hello = BigInt::parse_bytes(b"-22405534230753963835153736737", 10).unwrap();
 
     assert_eq!(format!("{:o}", a), "12");
     assert_eq!(format!("{:o}", hello), "-22062554330674403566756233062041");
@@ -998,7 +1200,7 @@ fn test_octal() {
 #[test]
 fn test_display() {
     let a = BigInt::parse_bytes(b"A", 16).unwrap();
-    let hello = BigInt::parse_bytes("-22405534230753963835153736737".as_bytes(), 10).unwrap();
+    let hello = BigInt::parse_bytes(b"-22405534230753963835153736737", 10).unwrap();
 
     assert_eq!(format!("{}", a), "10");
     assert_eq!(format!("{}", hello), "-22405534230753963835153736737");
@@ -1021,25 +1223,6 @@ fn test_negative_shr() {
     assert_eq!(BigInt::from(-3) >> 2, BigInt::from(-1));
 }
 
-#[test]
-#[cfg(feature = "rand")]
-fn test_random_shr() {
-    use rand::Rng;
-    use rand::distributions::Standard;
-    let mut rng = rand::thread_rng();
-
-    for p in rng.sample_iter::<i64, _>(&Standard).take(1000) {
-        let big = BigInt::from(p);
-        let bigger = &big << 1000;
-        assert_eq!(&bigger >> 1000, big);
-        for i in 0..64 {
-            let answer = BigInt::from(p >> i);
-            assert_eq!(&big >> i, answer);
-            assert_eq!(&bigger >> (1000 + i), answer);
-        }
-    }
-}
-
 #[test]
 fn test_iter_sum() {
     let result: BigInt = FromPrimitive::from_isize(-1234567).unwrap();
@@ -1053,8 +1236,8 @@ fn test_iter_sum() {
         FromPrimitive::from_i32(-7).unwrap(),
     ];
 
-    assert_eq!(result, data.iter().sum());
-    assert_eq!(result, data.into_iter().sum());
+    assert_eq!(result, data.iter().sum::<BigInt>());
+    assert_eq!(result, data.into_iter().sum::<BigInt>());
 }
 
 #[test]
@@ -1066,11 +1249,14 @@ fn test_iter_product() {
         FromPrimitive::from_i32(-1004).unwrap(),
         FromPrimitive::from_i32(1005).unwrap(),
     ];
-    let result = data.get(0).unwrap() * data.get(1).unwrap() * data.get(2).unwrap()
-        * data.get(3).unwrap() * data.get(4).unwrap();
+    let result = data.get(0).unwrap()
+        * data.get(1).unwrap()
+        * data.get(2).unwrap()
+        * data.get(3).unwrap()
+        * data.get(4).unwrap();
 
-    assert_eq!(result, data.iter().product());
-    assert_eq!(result, data.into_iter().product());
+    assert_eq!(result, data.iter().product::<BigInt>());
+    assert_eq!(result, data.into_iter().product::<BigInt>());
 }
 
 #[test]
@@ -1078,17 +1264,143 @@ fn test_iter_sum_generic() {
     let result: BigInt = FromPrimitive::from_isize(-1234567).unwrap();
     let data = vec![-1000000, -200000, -30000, -4000, -500, -60, -7];
 
-    assert_eq!(result, data.iter().sum());
-    assert_eq!(result, data.into_iter().sum());
+    assert_eq!(result, data.iter().sum::<BigInt>());
+    assert_eq!(result, data.into_iter().sum::<BigInt>());
 }
 
 #[test]
 fn test_iter_product_generic() {
     let data = vec![1001, -1002, 1003, -1004, 1005];
-    let result = data[0].to_bigint().unwrap() * data[1].to_bigint().unwrap()
-        * data[2].to_bigint().unwrap() * data[3].to_bigint().unwrap()
+    let result = data[0].to_bigint().unwrap()
+        * data[1].to_bigint().unwrap()
+        * data[2].to_bigint().unwrap()
+        * data[3].to_bigint().unwrap()
         * data[4].to_bigint().unwrap();
 
-    assert_eq!(result, data.iter().product());
-    assert_eq!(result, data.into_iter().product());
+    assert_eq!(result, data.iter().product::<BigInt>());
+    assert_eq!(result, data.into_iter().product::<BigInt>());
+}
+
+#[test]
+fn test_pow() {
+    let one = BigInt::from(1i32);
+    let two = BigInt::from(2i32);
+    let four = BigInt::from(4i32);
+    let eight = BigInt::from(8i32);
+    let minus_two = BigInt::from(-2i32);
+    macro_rules! check {
+        ($t:ty) => {
+            assert_eq!(Pow::pow(&two, 0 as $t), one);
+            assert_eq!(Pow::pow(&two, 1 as $t), two);
+            assert_eq!(Pow::pow(&two, 2 as $t), four);
+            assert_eq!(Pow::pow(&two, 3 as $t), eight);
+            assert_eq!(Pow::pow(&two, &(3 as $t)), eight);
+            assert_eq!(Pow::pow(&minus_two, 0 as $t), one, "-2^0");
+            assert_eq!(Pow::pow(&minus_two, 1 as $t), minus_two, "-2^1");
+            assert_eq!(Pow::pow(&minus_two, 2 as $t), four, "-2^2");
+            assert_eq!(Pow::pow(&minus_two, 3 as $t), -&eight, "-2^3");
+        };
+    }
+    check!(u8);
+    check!(u16);
+    check!(u32);
+    check!(u64);
+    check!(usize);
+
+    let pow_1e10000 = BigInt::from(10u32).pow(10_000_u32);
+    let manual_1e10000 = repeat(10u32).take(10_000).product::<BigInt>();
+    assert!(manual_1e10000 == pow_1e10000);
+}
+
+#[test]
+fn test_bit() {
+    // 12 = (1100)_2
+    assert!(!BigInt::from(0b1100u8).bit(0));
+    assert!(!BigInt::from(0b1100u8).bit(1));
+    assert!(BigInt::from(0b1100u8).bit(2));
+    assert!(BigInt::from(0b1100u8).bit(3));
+    assert!(!BigInt::from(0b1100u8).bit(4));
+    assert!(!BigInt::from(0b1100u8).bit(200));
+    assert!(!BigInt::from(0b1100u8).bit(u64::MAX));
+    // -12 = (...110100)_2
+    assert!(!BigInt::from(-12i8).bit(0));
+    assert!(!BigInt::from(-12i8).bit(1));
+    assert!(BigInt::from(-12i8).bit(2));
+    assert!(!BigInt::from(-12i8).bit(3));
+    assert!(BigInt::from(-12i8).bit(4));
+    assert!(BigInt::from(-12i8).bit(200));
+    assert!(BigInt::from(-12i8).bit(u64::MAX));
+}
+
+#[test]
+fn test_set_bit() {
+    let mut x: BigInt;
+
+    // zero
+    x = BigInt::zero();
+    x.set_bit(200, true);
+    assert_eq!(x, BigInt::one() << 200);
+    x = BigInt::zero();
+    x.set_bit(200, false);
+    assert_eq!(x, BigInt::zero());
+
+    // positive numbers
+    x = BigInt::from_biguint(Plus, BigUint::one() << 200);
+    x.set_bit(10, true);
+    x.set_bit(200, false);
+    assert_eq!(x, BigInt::one() << 10);
+    x.set_bit(10, false);
+    x.set_bit(5, false);
+    assert_eq!(x, BigInt::zero());
+
+    // negative numbers
+    x = BigInt::from(-12i8);
+    x.set_bit(200, true);
+    assert_eq!(x, BigInt::from(-12i8));
+    x.set_bit(200, false);
+    assert_eq!(
+        x,
+        BigInt::from_biguint(Minus, BigUint::from(12u8) | (BigUint::one() << 200))
+    );
+    x.set_bit(6, false);
+    assert_eq!(
+        x,
+        BigInt::from_biguint(Minus, BigUint::from(76u8) | (BigUint::one() << 200))
+    );
+    x.set_bit(6, true);
+    assert_eq!(
+        x,
+        BigInt::from_biguint(Minus, BigUint::from(12u8) | (BigUint::one() << 200))
+    );
+    x.set_bit(200, true);
+    assert_eq!(x, BigInt::from(-12i8));
+
+    x = BigInt::from_biguint(Minus, BigUint::one() << 30);
+    x.set_bit(10, true);
+    assert_eq!(
+        x,
+        BigInt::from_biguint(Minus, (BigUint::one() << 30) - (BigUint::one() << 10))
+    );
+
+    x = BigInt::from_biguint(Minus, BigUint::one() << 200);
+    x.set_bit(40, true);
+    assert_eq!(
+        x,
+        BigInt::from_biguint(Minus, (BigUint::one() << 200) - (BigUint::one() << 40))
+    );
+
+    x = BigInt::from_biguint(Minus, (BigUint::one() << 200) | (BigUint::one() << 100));
+    x.set_bit(100, false);
+    assert_eq!(
+        x,
+        BigInt::from_biguint(Minus, (BigUint::one() << 200) | (BigUint::one() << 101))
+    );
+
+    x = BigInt::from_biguint(Minus, (BigUint::one() << 63) | (BigUint::one() << 62));
+    x.set_bit(62, false);
+    assert_eq!(x, BigInt::from_biguint(Minus, BigUint::one() << 64));
+
+    x = BigInt::from_biguint(Minus, (BigUint::one() << 200) - BigUint::one());
+    x.set_bit(0, false);
+    assert_eq!(x, BigInt::from_biguint(Minus, BigUint::one() << 200));
 }
